#!/usr/bin/env python

# Example 1: calculate the mean scalar wind speed of the first 10 layers
import pynacolada as pcd
from Scientific.IO import NetCDF
import numpy as np

fnin = '/home/hendrik/data/belgium_aq/rcm/aq09/stage1/int2lm/laf2009010100_urb_ahf.nc'
#print fnin
# fobjin = open(fnin,'rb')
fin = NetCDF.NetCDFFile(fnin,'r')
fnout = 'laf2009010100_out.nc'
#print fnout
# fobjout = open(fnout,'wb+''rlat')
fout = NetCDF.NetCDFFile(fnout,'w')
# input data definitions
datin =  [{'file': fin, \
           'varname': 'U', \
           'dsel': {'level' : range(30,40,1)}, \
           'daliases': { 'srlat':'rlat', 'srlon':'rlon' },\
          },\
          {'file': fin, \
           'varname':'V', \
           'dsel': {'level' : range(30,40,1)},
           'daliases': { 'srlat':'rlat', 'srlon':'rlon' },\
           }\
         ]
# output data definitions
datout = [{'file': fout, \
           'varname': 'u'}]
# function definition:
func = lambda U,V: np.array([np.mean(np.sqrt(U**2+V**2),axis=0 )])
dnamsel = ['level',]
pcd.pcd(func,dnamsel,datin,datout,appenddim=True)
print 'output data written to:',fnout




import pynacolada as pcd
import numpy as np
import datetime as dt
import sciproc as sp
from Scientific.IO import NetCDF

# purpose: More examples for pynacolada.pcd() defining more advanced functions

# example 1: calculate running time-average of a dataset.

# Note: this function is included in sciproc ('from sciproc import avgglide'), but here it is explicitly given as an example
def avgglide(x,avlen):
    xout = np.zeros_like(x)
    xwork = np.zeros_like(x)
    if np.mod(avlen,2) == 0:
        xwork[0] = np.nan
        xwork[:-1] = (x[1:] + x[:-1])/2.
    else:
        xwork = x
    lenxout = xout.shape[0]
    avlen2 = int(avlen/2.)
    xout[:avlen2] = np.nan
    xout[-avlen2:] = np.nan
    for i in range(0,avlen,1):
        xout[avlen2:(-avlen+avlen2)] = xout[avlen2:(-avlen+avlen2)] + xwork[i:(-avlen+i)]
    return xout/avlen



# now, apply weekly cycle to a netcdf file
ncin = NetCDF.NetCDFFile('input.nc','r')
ncglide = NetCDF.NetCDFFile('output_glide.nc','w')
# we can cycle through the netcdf variables that have the 'time'-dimension
for evar in ncin.variables:
    if ((evar not in ncin.dimensions) & ('time' in ncin.variables[evar].dimensions):
        pcd.pcd(lambda x: avgglide(x,168),('time',),[{'file' : ncin, 'varname': evar}],[{'file' : ncglide, 'varname': evar}])
ncglide.close(); print('data written to:', ncglide)
ncin.close()





# generated by ./ubuntubuild.sh
from distutils.core import setup

setup(
    name='pynacolada',
    version='0.2.8',
    author='H. Wouters',
    author_email='hendrikwout@gmail.com',
    packages=['pynacolada'],
    url='http://www.nowebsite.com',
    license='LICENSE.txt',
    description='easy and customized Processing huge amounts of gridded Climate Data.',
    long_description=open('README.md').read(),
)




import datetime as dt
import numpy as np

def nctypecode(dtype):
    # purose: netcdf-typecode from array-dtype
    if ((dtype == np.dtype('float32')) or (np.dtype == 'float32')):
        return 'f'
    elif ((dtype == np.dtype('float64')) or (np.dtype == 'float64')):
        return 'd'
    elif ((dtype == np.dtype('int32')) or (np.dtype == 'int32')):
        return 'i'
    elif ((dtype == np.dtype('int64')) or (np.dtype == 'int64')):
        return 'l'

def ncgetdatetime(ncin):
    ''' extract datetimes from the 'time' coordinate in ncin
       
        ncin: input netcdf file
       
        returns an array of datetimes
    '''
    tunits = getattr(ncin.variables['time'],'units').split()
    if tunits[0] == 'days':
        mul = 24.*3600.
    elif tunits[0] == 'hours':
        mul = 3600.
    elif tunits[0] == 'minutes':
        mul = 60.
    elif tunits[0] == 'seconds':
        mul = 1.
    else:
        raise Exception("no time conversion found for '"+tunits[0],+"'")
    
    try:
        refdat = dt.datetime.strptime(tunits[2]+' '+tunits[3],"%Y-%m-%d %H:%M:%S")
    except:
        refdat = dt.datetime.strptime(tunits[2]+' '+tunits[3],"%Y-%m-%d %H:%M")
    
    return [(refdat+ dt.timedelta(seconds=e*mul)) for e in ncin.variables['time'][:]]
    #return(multifunc(lambda x: refdat + timedelta(seconds=x[0]), [ncin.variables['time'][:]*mul],[False],[[0]])[0])





from numpy import *
from datetime import *
from matplotlib.dates import *

# get corresponding format for netcdf file
def ncgetformat(fmt):    
    if fmt == float64: #double
        return 'd'
    elif fmt == float32:
        return 'f'
    elif fmt == int:
        return 'i'
    elif fmt == long:
        return 'l'
    elif fmt == datetime:
        return 'd' 
    else:
        print (' WARNING: format '+fmt+' could not be discovered! Assuming that it is double.')
        return 'd'

def csv2netcdf(infile,outfile,sep=None,nalist = [''],formatlist=[],refdat = datetime(2000,1,1),tunits = 'hours',customnames=[]):
    '''
    purpose: convert a csv-file to a netcdf timeseries. 

    infile: reference to a filestream containing csv data (example: open('file.csv'))
    outfile: reference to a writable netcdf-file NetCDF.NetCDFFile('file.nc')
    nalist: list with characters that should be skipped. It will be assigned as nan
    formatlist: a list of pairs, in which one specifies the format for each csv-column. The first element
    of a pair is the data type, the second element is the data format (e.g. like '%Y/%m/%d for a date)?.
    Example: ((datetime,'%Y/%m/%d',(double,nan))
    tunits: the unit for the 'time' coordinate (default: 'hours')
    refdat: reference date. (will be used to make the time dimension e.g. 'hours' since refdat)
    ''' 

    # todo: 
    tell = infile.tell()
    leninfile = len(infile.readlines()) - 1
    infile.seek(tell)
    colnames = infile.readline().replace('\n','').split(sep)

    for customname in customnames:
        colnames[customname[0]] = customname[1]

    # # workaround ncview crash for underscores
    # for colname in colnames:
    #     colname.replace('_','')
    #formatlist.extend(repeat((nan,nan),len(colnames) - len(formatlist)))
    for i in range(len(colnames) - len(formatlist)):
        formatlist.append((double,nan))

    outfile.createDimension('time',leninfile)
    outfile.createVariable('time','d',('time',))
    outfile.createVariable('datetime','d',('time',))
    if (tunits not in nalist):
        setattr(outfile.variables['time'],\
                'units', \
                tunits +' since '+ datetime.strftime(refdat,'%Y-%m-%d %H:%M:%S'))

    for icn,ecn in enumerate(colnames):
        outfile.createVariable(ecn,ncgetformat(formatlist[icn][0]),('time',))
    for iline,eline in enumerate(infile):
        currline = eline.replace('\n','').split(sep)
        colnum = min(len(currline),len(colnames))
        for icn in range(colnum):
            ecn = colnames[icn]
            if currline[icn] not in nalist:
                if formatlist[icn][0] == datetime:
                    outfile.variables[ecn][iline] = date2num(datetime.strptime(currline[icn],formatlist[icn][1]))
                else:
                    try:
                        outfile.variables[ecn][iline] = formatlist[icn][0](currline[icn])
                    except:
                        outfile.variables[ecn][iline] = nan
            else:
                outfile.variables[ecn][iline] = nan

        uzformatlist = zip(*formatlist)
        # get date of string
        dt = refdat
        dtype = datetime
        if dtype in uzformatlist[0]:
            icn = uzformatlist[0].index(dtype)
            if currline[icn] not in nalist:
                dt = datetime.strptime(currline[icn],uzformatlist[1][icn])
        dtype = date
        if dtype in uzformatlist[0]:
            icn = uzformatlist[0].index(dtype)
            if currline[icn] not in nalist:
                dt = datetime.strptime(currline[icn],uzformatlist[1][icn])

        # tbi loop over every 'daytime' type and add them together
        dtype = 'daytime'
        if dtype in uzformatlist[0]:
            icn = uzformatlist[0].index(dtype)
            if currline[icn] not in nalist:
                currval = double(currline[icn])
                sec = 0.
                if uzformatlist[icn][1] == 'HMint':
                    sec = (int(currval/100.)*60. + (currval - int(currval/100.)*100.))*60.
                dt = dt + timedelta(seconds = sec)

        # output to netcdf
        outfile.variables['datetime'][iline] = date2num(dt)

        if tunits == 'hours':
            outfile.variables['time'][iline] = \
                    (dt-refdat).total_seconds()/3600.
        if tunits == 'days':
            outfile.variables['time'][iline] = \
                    (dt-refdat).total_seconds()/3600./24.
        if tunits == 'seconds':
            outfile.variables['time'][iline] = \
                    (dt-refdat).total_seconds()
        if tunits == 'minutes':
            outfile.variables['time'][iline] = \
                    (dt-refdat).total_seconds()/60.

def ncwritedatetime(ncfile,dt,tunits = None, refdat = None):
    # purpose: write given time coordinate to ncdf file
    # dt: array of datetimes
    # tunits: time unit (can be either 'hour', 'days', 'minutes','seconds',
    # refdat: reference date from which the timesteps are represented

    if refdat == None: refdat = dt[0]
    if 'time' not in ncfile.dimensions:
        ncfile.createDimension('time',len(dt))
    if 'time' not in ncfile.variables:
        ncfile.createVariable('time','d',('time',))

    if tunits == None:
        try:
            if (dt[1] - dt[0]) == timedelta(years=1): tunits = 'years'
            elif (dt[1] - dt[0]) == timedelta(days=1): tunits = 'days'
            elif (dt[1] - dt[0]) == timedelta(hours=1): tunits = 'hours'
            elif (dt[1] - dt[0]) == timedelta(minutes=1): tunits = 'minutes'
            elif (dt[1] - dt[0]) == timedelta(hours=1): tunits = 'seconds'
            else: tunits = 'hours'
        except:
            tunits = 'hours'

    setattr(ncfile.variables['time'],\
            'units', \
             tunits +' since '+ datetime.strftime(refdat,'%Y-%m-%d %H:%M:%S'))


    for idt,edt in enumerate(dt):
        if tunits == 'hours':
            ncfile.variables['time'][idt] = \
                    (edt-refdat).total_seconds()/3600.
        if tunits == 'days':
            ncfile.variables['time'][idt] = \
                    (edt-refdat).total_seconds()/3600./24.
        if tunits == 'seconds':
            ncfile.variables['time'][idt] = \
                    (edt-refdat).total_seconds()
        if tunits == 'minutes':
            ncfile.variables['time'][idt] = \
                    (edt-refdat).total_seconds()/60.

